﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("Directed Path", "")]
  [StorableType("4FD1A99C-7858-4AB9-916F-FEE00B4FD66D")]
  public class DirectedPath<T> : Item where T : class, IDeepCloneable {
    [Storable]
    private List<List<Tuple<T, double>>> paths;
    public IEnumerable<List<Tuple<T, double>>> Paths {
      get { return paths ?? new List<List<Tuple<T, double>>>(); }
    }

    [StorableConstructor]
    protected DirectedPath(StorableConstructorFlag _) : base(_) { }
    protected DirectedPath(DirectedPath<T> original, Cloner cloner)
      : base(original, cloner) {
      if (original.paths != null)
        paths = original.paths.Select(x => x.Select(y => Tuple.Create(cloner.Clone(y.Item1), y.Item2)).ToList()).ToList();
    }
    public DirectedPath() { }

    public bool IsEmpty {
      get { return paths == null || paths.Count == 0; }
    }

    public void AddPath(IEnumerable<Tuple<T, double>> path) {
      if (paths == null) paths = new List<List<Tuple<T, double>>>();
      paths.Add(path.ToList());
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DirectedPath<T>(this, cloner);
    }
  }
}
